package id.encrypt.jwtson
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.fusionauth.jwt.Signer
import io.fusionauth.jwt.Verifier
import io.fusionauth.jwt.domain.JWT
import io.fusionauth.jwt.hmac.HMACSigner
import io.fusionauth.jwt.hmac.HMACVerifier
import org.json.JSONObject
import java.lang.Exception

/*
 * Dayona
 * Created on Thu Jun 02 2022
 * email : dayonaart@gmail.com
 * Copyright (c) 2022
 */


/** JwtsonPlugin */
class JwtsonPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "jwtson")
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
      val arg = call.arguments as HashMap<*, *>
    when (call.method) {
        "generatePayload" -> {
          result.success( generatePayload(arg["key"] as String,arg["data"] as String))
        }
        "verifyJwtToken" -> {
            result.success(verifyJWT(arg["data"] as String,arg["key"] as String))
        }
        "decodeJWT" -> {
            result.success(decodeJWT(arg["key"] as String,arg["data"] as String))
        }
        else -> {
          result.notImplemented()
        }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

    private fun generatePayload(key: String, data: String): String {
        val signer: Signer = HMACSigner.newSHA256Signer(key)
        val jwt = JWT().addClaim("data", data)
        return JWT.getEncoder().encode(jwt, signer)
    }

    private fun verifyJWT(signedJWT:String,key:String):Boolean{
        return try {
            val verifier: Verifier = HMACVerifier.newVerifier(key)
            val jwt = JWT.getDecoder().decode(signedJWT, verifier)
            true
        }catch (e:Exception){
            false
        }
    }
    private fun decodeJWT(key:String,encodedJWT:String):String{
        return try {
            val verifier: Verifier = HMACVerifier.newVerifier(key)
            val jwt = JWT.getDecoder().decode(encodedJWT, verifier)
            print(jwt.getOtherClaims().toString())
           val json= JSONObject(jwt.getOtherClaims())
            "$json"
        }catch (e:Exception){
            "Error Decode JWT Token"
        }
    }
}
