import Flutter
import UIKit
import SwiftJWT
import CryptoKit

/**
 * Dayona
 * Created on Thu Jun 02 2022
 * email : dayonaart@gmail.com
 * Copyright (c) 2022
 */


public class SwiftJwtsonPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "jwtson", binaryMessenger: registrar.messenger())
        let instance = SwiftJwtsonPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "generatePayload" {
            let arg = call.arguments as! NSDictionary
            result(generatePayload(key:arg["key"] as! String,data: arg["data"] as! String))
        }else if call.method == "verifyJwtToken" {
            let arg = call.arguments as! NSDictionary
            result(verifyJWT(signedJWT:arg["data"] as! String, key:arg["key"] as! String))
        }else if call.method == "decodeJWT" {
            let arg = call.arguments as! NSDictionary
            result(decodeJWT(key:arg["key"] as! String, encodedJWT:arg["data"] as! String))
        }  else {
            result(FlutterMethodNotImplemented)
        }

    }
}
private func generatePayload(key:String,data:String) -> String{
    let payloadJSONData = data.data(using: .utf8)
    let payloadBase64String = payloadJSONData!.urlSafeBase64EncodedString()
    let secret = key
    let privateKey = SymmetricKey(data: Data(secret.utf8))
    let headerJSONData = try! JSONEncoder().encode(Header())
    let headerBase64String = headerJSONData.urlSafeBase64EncodedString()
    let toSign = Data((headerBase64String + "." + payloadBase64String).utf8)
    let signature = HMAC<SHA256>.authenticationCode(for: toSign, using: privateKey)
    let signatureBase64String = Data(signature).urlSafeBase64EncodedString()
    let token = [headerBase64String, payloadBase64String, signatureBase64String].joined(separator: ".")
    return token
}


private func verifyJWT(signedJWT:String,key:String) -> Bool{
    let jwtVerifier = JWTVerifier.hs256(key:  Data(key.utf8))
    return JWT<Payload>.verify(signedJWT, using: jwtVerifier)
}

private func decodeJWT(key:String,encodedJWT:String)-> [String:Any]{
    let jwtVerifier = JWTVerifier.hs256(key:  Data(key.utf8))
    let jwtDecoder = JWTDecoder(jwtVerifier: jwtVerifier)
//    let jwt = try? jwtDecoder.decode(JWT<Payload>.s, fromString: encodedJWT)
//    let json = ["data":jwt?.data ?? "Error Decode JWT token"] as [String : Any]
//    let jsonData = try! JSONSerialization.data(withJSONObject: json, options: JSONSerialization.WritingOptions.prettyPrinted)
//    let jsonString = NSString(data: jsonData, encoding: String.Encoding.utf8.rawValue) as? String
    return ["json":""]

}

//struct MyClaims: Claims {
//    let data: String
////    let exp: Date
//}

extension Date {
    var millisecondsSince1970: Int64 {
        Int64((self.timeIntervalSince1970 * 1000.0).rounded())
    }

    init(milliseconds: Int64) {
        self = Date(timeIntervalSince1970: TimeInterval(milliseconds) / 1000)
    }
}

private func convertToDictionary(text: String) -> [String: Any]? {
    if let data = text.data(using: .utf8) {
        do {
            return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
        } catch {
            print(error.localizedDescription)
        }
    }
    return nil
}

extension Data {
    func urlSafeBase64EncodedString() -> String {
        return base64EncodedString()
            .replacingOccurrences(of: "+", with: "-")
            .replacingOccurrences(of: "/", with: "_")
            .replacingOccurrences(of: "=", with: "")
    }
}
extension Encodable {
  var dictionary: [String: Any]? {
    guard let data = try? JSONEncoder().encode(self) else { return nil }
    return (try? JSONSerialization.jsonObject(with: data, options: .allowFragments)).flatMap { $0 as? [String: Any] }
  }
}
extension Encodable {
  func asDictionary() throws -> [String: Any] {
    let data = try JSONEncoder().encode(self)
    guard let dictionary = try JSONSerialization.jsonObject(with: data, options: .allowFragments) as? [String: Any] else {
      throw NSError()
    }
    return dictionary
  }
}
struct Header: Encodable {
    let alg = "HS256"
    let typ = "JWT"
}

struct Payload : Claims{
    let data:[String:String]
}
