import 'package:dart_jsonwebtoken/dart_jsonwebtoken.dart';
import 'package:flutter/material.dart';
import 'package:jwtson/jwtson_method_channel.dart';
import 'jwtson_platform_interface.dart';

/*
 * Dayona
 * Created on Thu Jun 02 2022
 * email : dayonaart@gmail.com
 * Copyright (c) 2022
 */

class Jwtson {
  /// Generate And Sign JWT TOKEN
  /// - sign key and data must not be null
  Future<String?> generateToken(
      {@required String? key, @required Map<String, dynamic>? data}) async {
    assert(key != null && data != null);
    // Create a json web token
    final jwt = JWT(data);
    // Sign it (default with HS256 algorithm)
    return jwt.sign(SecretKey(key!));
    // return JwtsonPlatform.instance.generateToken(key, jsonEncode(data));
  }

  /// Decode And with sign key JWT TOKEN
  /// - sign key and data ( JWT token ) must not be null
  Future<PayloadResult?> decodeJwtToken({@required String? key, @required String? data}) async {
    assert(key != null && data != null);
    try {
      // Verify a token
      final jwt = JWT.verify(data!, SecretKey(key!));
      return PayloadResult(data: jwt.payload);
      // return JwtsonPlatform.instance.decodeJwtToken(key, data);
    } on JWTExpiredError {
      return PayloadResult(data: "jwt expired");
    } on JWTError catch (ex) {
      return PayloadResult(data: ex.message);
    }
  }
}
